unit hiFileStream;

{$WARNINGS OFF}

interface

uses Kol,Share,windows,Debug;

type
  THIFileStream = class(TDebug)
  private
    Fs: PStream;
    procedure Open(var _Data: TData);
    function GetErrorCode: Integer;
  public
    _prop_FileName: string;
    _prop_Mode: Byte;
    _prop_ShareMode: Byte;
    _prop_CreationFlag: Byte;
    _prop_AutoCopy: Boolean;
    

    _data_FileName: THI_Event;
    _event_onLoad: THI_Event;
    _event_onError: THI_Event;

    procedure _work_doOpen(var _Data: TData; Index: Word);
    procedure _work_doClose(var _Data: TData; Index: Word);
    procedure _work_doCopyFromStream(var _Data: TData; Index: Word);
    procedure _work_doPosition(var _Data: TData; Index: Word);
    procedure _work_doSetSize(var _Data: TData; Index: Word);
    
    procedure _work_doMode(var _Data: TData; Index: Word);
    procedure _work_doShareMode(var _Data: TData; Index: Word);
    procedure _work_doCreationFlag(var _Data: TData; Index: Word);
    
    procedure _var_Stream(var _Data: TData; Index: Word);
    procedure _var_Size(var _Data: TData; Index: Word);
    procedure _var_Position(var _Data: TData; Index: Word);
  end;

implementation

procedure THIFileStream._work_doOpen;
begin
  Open(_Data);
  if Fs.Handle = INVALID_HANDLE_VALUE then
    _hi_OnEvent(_event_onError, GetErrorCode)
  else
    _hi_CreateEvent(_Data,@_event_onLoad,Fs);
end;

function THIFileStream.GetErrorCode;
var LastErr:DWORD;
begin
  LastErr := GetLastError;
  case LastErr of
    2,3: result := 0; //     -   
    32: result := 1; //      -    
    80: result := 2; //     -   
    5: result := 3; //   
    87: result := 4; // CreationFlag = TruncateExisting    Mode = Read
  else 
    result := LastErr;
  end;
end;

procedure THIFileStream.Open;
var fn:string; Flags:DWORD;
begin
  Free_And_Nil(Fs);
  fn := ReadString(_Data,_data_FileName,_prop_FileName);
  
  case _prop_Mode of 
    0: Flags := ofOpenRead;
    1: Flags := ofOpenWrite;
    2: Flags := ofOpenReadWrite;
  end;
  
  Flags := Flags + _prop_ShareMode;
  
  if _prop_CreationFlag = 0 then  // CreationFlag = Auto
    if _prop_Mode = 0 then Flags := Flags + ofOpenExisting else Flags := Flags + ofOpenAlways
    {case _prop_Mode of   
      0: Flags := Flags + ofOpenExisting;
      //1, 2: Flags := Flags + ofOpenAlways;
      //2: if FileExists(fn) then Flags := Flags + ofOpenExisting else Flags := Flags + ofCreateAlways;
    end}    
  else Flags := Flags + (_prop_CreationFlag * $100);
  
  Fs := NewFileStream(fn, Flags);                     
end;

procedure THIFileStream._work_doClose;
begin
  Free_And_Nil(Fs);
end;

procedure THIFileStream._work_doCopyFromStream;
var s:PStream;O:boolean;
begin
 
  O := (not Assigned(Fs))and _prop_AutoCopy;
  if O then Open(_Data);
  if Assigned(Fs) and _isStream(_Data) then begin
    if _prop_Mode = 0 then
      MessageBox(0,'Please set Mode property to Write','File stream Error',MB_OK)
    else 
    begin
      s := ToStream(_Data);
      s.Position := 0;
      if (_prop_Mode=2)and O then Fs.Position := Fs.Size;
      Stream2Stream(Fs,s,s.Size);
    end;
  end;
  if O then Free_And_Nil(Fs);
end;

procedure THIFileStream._work_doPosition;
var
  NewPos: Integer;
begin
  if Assigned(Fs) then
  begin
    NewPos := ToInteger(_Data);
    //     2  4 
    if NewPos < 0 then
    begin
      Fs.Seek($7FFFFFFF, spBegin); //   2   
      Fs.Seek(Cardinal(NewPos)-$7FFFFFFF, spCurrent); //        
    end
    else
      Fs.Seek(NewPos, spBegin);
  end;
end;

procedure THIFileStream._work_doSetSize;
begin
  if Assigned(Fs) then Fs.Size := ToInteger(_Data);
end;

procedure THIFileStream._var_Stream;
begin
  dtNull(_Data);
  if Assigned(Fs) then dtStream(_Data,Fs)
end;

procedure THIFileStream._var_Size;
begin
  dtNull(_Data);
  if Assigned(Fs) then dtInteger(_Data,Fs.Size)
end;

procedure THIFileStream._var_Position;
begin
  dtNull(_Data);
  if Assigned(Fs) then dtInteger(_Data,Fs.Position)
end;

procedure THIFileStream._work_doMode(var _Data: TData; Index: Word);
begin
  _prop_Mode := byte(ToInteger(_Data));
end;

procedure THIFileStream._work_doShareMode(var _Data: TData; Index: Word);
begin
  _prop_ShareMode := byte(ToInteger(_Data));
end;

procedure THIFileStream._work_doCreationFlag(var _Data: TData; Index: Word);
begin
  _prop_CreationFlag := byte(ToInteger(_Data));
end;


{$WARNINGS ON}

end.
